@@ -1,49 +1,43 @@
-/**************************************************************************
-**
-** This file is part of the KDE Frameworks
-**
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-** Copyright (c) 2019 David Hallas <david@davidhallas.dk>
-**
-** GNU Lesser General Public License Usage
-**
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
-**
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+/*
+    This file is part of the KDE Frameworks
 
+    SPDX-FileCopyrightText: 2011 Nokia Corporation and/or its subsidiary(-ies).
+    SPDX-FileCopyrightText: 2019 David Hallas <david@davidhallas.dk>
+
+    SPDX-License-Identifier: LGPL-2.1-only WITH Qt-LGPL-exception-1.1 OR LicenseRef-Qt-Commercial
+*/
+
+/*
+ * Implementation notes:
+ *
+ * This file implements KProcessInfo and KProcessInfoList via Linux /proc
+ * **or** via ps(1). If there's no /proc, it falls back to ps(1), usually.
+ *
+ * Although the code contains #ifdefs for FreeBSD (e.g. for ps(1) command-
+ * line arguments), FreeBSD should never use this code, only the
+ * procstat-based code in `kprocesslist_unix_procstat.cpp`.
+ */
+
+#include "kcoreaddons_debug.h"
 #include "kprocesslist.h"
 
-#include <QProcess>
+#include <QDebug>
 #include <QDir>
+#include <QProcess>
+
+#ifdef Q_OS_FREEBSD
+#error This KProcessInfo implementation is not supported on FreeBSD (use procstat)
+#endif
 
 using namespace KProcessList;
 
-namespace {
-
+namespace
+{
 bool isUnixProcessId(const QString &procname)
 {
-    for (int i = 0; i != procname.size(); ++i) {
-        if (!procname.at(i).isDigit())
-            return false;
-    }
-    return true;
+    return std::none_of(procname.cbegin(), procname.cend(), [](const QChar ch) {
+        return !ch.isDigit();
+    });
 }
 
 // Determine UNIX processes by running ps
@@ -51,21 +45,27 @@ KProcessInfoList unixProcessListPS()
 {
     KProcessInfoList rc;
     QProcess psProcess;
-    const QStringList args {
+    const QStringList args{
         QStringLiteral("-e"),
         QStringLiteral("-o"),
 #ifdef Q_OS_MAC
-    // command goes last, otherwise it is cut off
+        // command goes last, otherwise it is cut off
         QStringLiteral("pid state user comm command"),
 #else
         QStringLiteral("pid,state,user,comm,cmd"),
 #endif
     };
     psProcess.start(QStringLiteral("ps"), args);
-    if (!psProcess.waitForStarted())
+    if (!psProcess.waitForStarted()) {
+        qCWarning(KCOREADDONS_DEBUG) << "Failed to execute ps" << args;
         return rc;
+    }
     psProcess.waitForFinished();
-    QByteArray output = psProcess.readAllStandardOutput();
+    const QByteArray output = psProcess.readAllStandardOutput();
+    const QByteArray errorOutput = psProcess.readAllStandardError();
+    if (!errorOutput.isEmpty()) {
+        qCWarning(KCOREADDONS_DEBUG) << "ps said" << errorOutput;
+    }
     // Split "457 S+   /Users/foo.app"
     const QStringList lines = QString::fromLocal8Bit(output).split(QLatin1Char('\n'));
     const int lineCount = lines.size();
@@ -75,15 +75,16 @@ KProcessInfoList unixProcessListPS()
         // we can't just split on blank as the process name might
         // contain them
         const int endOfPid = line.indexOf(blank);
-        const int endOfState = line.indexOf(blank, endOfPid+1);
-        const int endOfUser = line.indexOf(blank, endOfState+1);
-        const int endOfName = line.indexOf(blank, endOfUser+1);
+        const int endOfState = line.indexOf(blank, endOfPid + 1);
+        const int endOfUser = line.indexOf(blank, endOfState + 1);
+        const int endOfName = line.indexOf(blank, endOfUser + 1);
 
         if (endOfPid >= 0 && endOfState >= 0 && endOfUser >= 0) {
-            qint64 pid = line.leftRef(endOfPid).toUInt();
-            QString user = line.mid(endOfState+1, endOfUser-endOfState-1);
-            QString name = line.mid(endOfUser+1, endOfName-endOfUser-1);
-            QString command = line.right(line.size()-endOfName-1);
+            const qint64 pid = line.leftRef(endOfPid).toUInt();
+
+            QString user = line.mid(endOfState + 1, endOfUser - endOfState - 1);
+            QString name = line.mid(endOfUser + 1, endOfName - endOfUser - 1);
+            QString command = line.right(line.size() - endOfName - 1);
             rc.push_back(KProcessInfo(pid, command, name, user));
         }
     }
@@ -91,23 +92,24 @@ KProcessInfoList unixProcessListPS()
     return rc;
 }
 
-bool getProcessInfo(const QString& procId, KProcessInfo& processInfo)
+bool getProcessInfo(const QString &procId, KProcessInfo &processInfo)
 {
-    if (!isUnixProcessId(procId))
+    if (!isUnixProcessId(procId)) {
         return false;
-#ifdef Q_OS_FREEBSD
-    QString statusFileName(QStringLiteral("/status"));
-#else
+    }
     QString statusFileName(QStringLiteral("/stat"));
-#endif
     QString filename = QStringLiteral("/proc/");
     filename += procId;
     filename += statusFileName;
     QFile file(filename);
-    if (!file.open(QIODevice::ReadOnly))
-        return false;     // process may have exited
+    if (!file.open(QIODevice::ReadOnly)) {
+        return false; // process may have exited
+    }
 
     const QStringList data = QString::fromLocal8Bit(file.readAll()).split(QLatin1Char(' '));
+    if (data.length() < 2) {
+        return false;
+    }
     qint64 pid = procId.toUInt();
     QString name = data.at(1);
     if (name.startsWith(QLatin1Char('(')) && name.endsWith(QLatin1Char(')'))) {
@@ -152,10 +154,12 @@ bool getProcessInfo(const QString& procId, KProcessInfo& processInfo)
 KProcessInfoList KProcessList::processInfoList()
 {
     const QDir procDir(QStringLiteral("/proc/"));
-    if (!procDir.exists())
+    if (!procDir.exists()) {
         return unixProcessListPS();
+    }
+    const QStringList procIds = procDir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
     KProcessInfoList rc;
-    const QStringList procIds = procDir.entryList();
+    rc.reserve(procIds.size());
     for (const QString &procId : procIds) {
         KProcessInfo processInfo;
         if (getProcessInfo(procId, processInfo)) {
@@ -165,6 +169,11 @@ KProcessInfoList KProcessList::processInfoList()
     return rc;
 }
 
+// Determine UNIX process by reading "/proc".
+//
+// TODO: Use ps if "/proc" does not exist or is bogus; use code
+//       from unixProcessListPS() but add a `-p pid` argument.
+//
 KProcessInfo KProcessList::processInfo(qint64 pid)
 {
     KProcessInfo processInfo;
